# Detectors

[solution_name] uses detectors to find and extract dependencies from all supported package managers.

Each package manager ecosystem is assigned a detector type. Each detector type may have multiple methods used to extract dependencies.

## Detector Search

Detectors first check to see if they apply to your project by looking for hints such as files that exist in your project directory
or properties you have set.

By default, detectors only search the project directory. In some cases, such as when your project contains sub-projects,
or when package manager files reside in sub-directories, you may need to tell [solution_name] to search sub-directories
by increasing the detector search depth. For more information, refer to [detector search depth](../properties/configuration/paths.md#detector-search-depth).

Detectors then check that your environment is extractable, meaning you have all the relevant executables such as NPM or a Gradle wrapper, and all relevant downloads are present or available, such as the Docker or NuGet inspector.

Inspectors are used by detectors when the package manager requires an integration or embedded plugin to work. For example, Gradle uses an inspector as a plugin that executes a custom task. Most detectors do not require an inspector.

Finally, detectors perform their extraction to find your dependencies. This may require, but is not limited to, running executables, performing builds, parsing files, and communicating with web services.

## [detector_cascade] and accuracy

[detector_cascade] and accuracy capabilities together replace the previous (pre-[solution_name] 8) distinction between "build mode" and "buildless mode",
and aim to provide a more user-friendly way to get the best results possible, while ensuring that you are alterted (via a [solution_name] failure)
if your accuracy requirements are not met.

In the table below, most Detector Types have a single Entry Point. For those Detector Types, the Entry Point column can be ignored.

There are a few Detector Types for which multiple Entry Points are defined.
When multipe Entry Points are defined for a Detector Type,
they exist to support the relatively rare need to define different nesting rules
within the same Detector Type for different scenarios.
(Nesting rules can usually be, and usually are, applied at the Detector Type level.)
For example: Detect should ignore XCODE project files that are nested inside an XCODE project that it has already processed.
Entry Points provide the mechanism by which more nuanced nesting rules such as this are applied.

Each Entry Point has one or more Detectors. Detectors are attempted in the order listed until one applies and succeeds.
If none succeed, [solution_name] proceeds to the next Entry Point (if there is one) for the Detector Type.

See [Detector cascade and accuracy](../downloadingandrunning/detectorcascade.md) for additional information on [detector_cascade]..

|Detector Type|Entry Point|Detector|Language|Forge|Requirements|Accuracy|
|---|---|---|---|---|---|---|
|**BITBAKE**|||||||
||Bitbake CLI||||||
||| Bitbake CLI |various|YOCTO | Properties: Package names. File: build env script. Executable: bash|HIGH|
|**CARGO**|||||||
||Cargo Lock||||||
||| Cargo Lock |Rust|crates | Files: Cargo.lock, Cargo.toml|HIGH|
|**CARTHAGE**|||||||
||Carthage Lock||||||
||| Carthage Lock |various|GitHub | Files: Cartfile, Cartfile.resolved|HIGH|
|**CLANG**|||||||
||Clang CLI||||||
||| Clang CLI |C or C++|Derived from the Linux distribution. | File: compile_commands.json. Executable: Linux package manager.|HIGH|
|**COCOAPODS**|||||||
||Pod Lock||||||
||| Pod Lock |Objective C|COCOAPODS and NPMJS | Files: Podfile.lock|HIGH|
|**CONAN**|||||||
||Conan Lock||||||
||| Conan Lock |C/C++|conan | Files: conan.lock.|HIGH|
||| Conan CLI |C/C++|conan | Files: conanfile.txt or conanfile.py. Executable: conan.|HIGH|
||Conan CLI||||||
||| Conan CLI |C/C++|conan | Files: conanfile.txt or conanfile.py. Executable: conan.|HIGH|
|**CONDA**|||||||
||Conda CLI||||||
||| Conda CLI |Python|Anaconda | File: environment.yml. Executable: conda.|HIGH|
|**CPAN**|||||||
||Cpan CLI||||||
||| Cpan CLI |Perl|CPAN | File: Makefile.PL. Executable: cpan.|HIGH|
|**CRAN**|||||||
||Packrat Lock||||||
||| Packrat Lock |R|CRAN | File: packrat.lock.|HIGH|
|**DART**|||||||
||Dart CLI||||||
||| Dart CLI |Dart|Dart | Files: pubspec.yaml, pubspec.lock. Executable: dart, flutter|HIGH|
||| Dart PubSpec Lock |Dart|Dart | Files: pubspec.yaml, pubspec.lock.|HIGH|
||Dart PubSpec Lock||||||
||| Dart PubSpec Lock |Dart|Dart | Files: pubspec.yaml, pubspec.lock.|HIGH|
|**GIT**|||||||
||Git||||||
||| Git |various|N/A | Directory: .git. (Executable: git).|HIGH|
||| Git Parse |various|N/A | Files: .git/config, .git/HEAD, .git/ORIG_HEAD.|HIGH|
||Git Parse||||||
||| Git Parse |various|N/A | Files: .git/config, .git/HEAD, .git/ORIG_HEAD.|HIGH|
|**GO_DEP**|||||||
||GoDep Lock||||||
||| GoDep Lock |Golang|GitHub | File: Gopkg.lock.|HIGH|
|**GO_GRADLE**|||||||
||GoGradle Lock||||||
||| GoGradle Lock |Golang|GitHub | File: gogradle.lock.|HIGH|
|**GO_MOD**|||||||
||GoMod CLI||||||
||| GoMod CLI |Golang|Go Modules | File: go.mod. Executable: go.|HIGH|
|**GO_VENDOR**|||||||
||Go Vendor||||||
||| Go Vendor |Golang|GitHub | File: vendor/vendor.json.|HIGH|
|**GO_VNDR**|||||||
||GoVndr CLI||||||
||| GoVndr CLI |Golang|GitHub | File: vendor.conf.|HIGH|
|**GRADLE**|||||||
||Gradle Native Inspector||||||
||| Gradle Native Inspector |various|Maven Central | File: build.gradle or build.gradle.kts. Executable: gradlew or gradle.|HIGH|
||| Gradle Project Inspector |various|Maven Central | File: build.gradle|LOW|
|**HEX**|||||||
||Rebar CLI||||||
||| Rebar CLI |Erlang|Hex | File: rebar.config. Executable: rebar3.|HIGH|
|**IVY**|||||||
||Ivy Build Parse||||||
||| Ivy Build Parse |various|Maven Central | File: ivy.xml, build.xml.|LOW|
|**LERNA**|||||||
||Lerna CLI||||||
||| Lerna CLI |Node JS|npmjs | File: package.json, and one of the following: package-lock.json, npm-shrinkwrap.json, yarn.lock.|HIGH|
|**MAVEN**|||||||
||Maven CLI||||||
||| Maven CLI |various|Maven Central | File: pom.xml. Executable: mvnw or mvn.|HIGH|
||| Maven Project Inspector |various|Maven Central | File: pom.xml.|LOW|
||Maven Wrapper CLI||||||
||| Maven Wrapper CLI |various|Maven Central | File: pom.groovy. Executable: mvnw or mvn.|HIGH|
||| Maven Project Inspector |various|Maven Central | File: pom.xml.|LOW|
|**NPM**|||||||
||NPM Shrinkwrap||||||
||| NPM Shrinkwrap |Node JS|npmjs | File: npm-shrinkwrap.json. Optionally for better results: package.json also.|HIGH|
||| NPM Package Lock |Node JS|npmjs | File: package-lock.json. Optionally for better results: package.json also.|HIGH|
||| NPM CLI |Node JS|npmjs | Files: node_modules, package.json. Executable: npm.|HIGH|
||| NPM Package Json Parse |Node JS|npmjs | File: package.json.|LOW|
||NPM Package Lock||||||
||| NPM Package Lock |Node JS|npmjs | File: package-lock.json. Optionally for better results: package.json also.|HIGH|
||| NPM CLI |Node JS|npmjs | Files: node_modules, package.json. Executable: npm.|HIGH|
||| NPM Package Json Parse |Node JS|npmjs | File: package.json.|LOW|
||NPM CLI||||||
||| NPM CLI |Node JS|npmjs | Files: node_modules, package.json. Executable: npm.|HIGH|
||| NPM Package Json Parse |Node JS|npmjs | File: package.json.|LOW|
||NPM Package Json Parse||||||
||| NPM Package Json Parse |Node JS|npmjs | File: package.json.|LOW|
|**NUGET**|||||||
||NuGet Solution Native Inspector||||||
||| NuGet Solution Native Inspector |C#|NuGet.org | File: a solution file with a .sln extension.|HIGH|
||| NuGet Project Inspector |C#|NuGet.org | File: a project file with one of the following extensions: .csproj, .sln|LOW|
||NuGet Project Native Inspector||||||
||| NuGet Project Native Inspector |C#|NuGet.org | File: a project file with one of the following extensions: .csproj, .fsproj, .vbproj, .asaproj, .dcproj, .shproj, .ccproj, .sfproj, .njsproj, .vcxproj, .vcproj, .xproj, .pyproj, .hiveproj, .pigproj, .jsproj, .usqlproj, .deployproj, .msbuildproj, .sqlproj, .dbproj, .rproj|HIGH|
||| NuGet Project Inspector |C#|NuGet.org | File: a project file with one of the following extensions: .csproj, .sln|LOW|
|**PACKAGIST**|||||||
||Composer Lock||||||
||| Composer Lock |PHP|Packagist.org | Files: composer.lock, composer.json.|HIGH|
|**PEAR**|||||||
||Pear CLI||||||
||| Pear CLI |PHP|Pear | Files: package.xml. Executable: pear.|HIGH|
|**PIP**|||||||
||Pipenv Lock||||||
||| Pipenv Lock |Python|PyPi | Files: Pipfile or Pipfile.lock. Executables: python or python3, and pipenv.|HIGH|
||| PIP Native Inspector |Python|Pypi | A setup.py file, or one or more requirements.txt files. Executables: python and pip, or python3 and pip3.|HIGH|
||| Pipfile Lock |Python|PyPi | Files: Pipfile, Pipfile.lock|HIGH|
||PIP Native Inspector||||||
||| PIP Native Inspector |Python|Pypi | A setup.py file, or one or more requirements.txt files. Executables: python and pip, or python3 and pip3.|HIGH|
||| Pipfile Lock |Python|PyPi | Files: Pipfile, Pipfile.lock|HIGH|
||Pipfile Lock||||||
||| Pipfile Lock |Python|PyPi | Files: Pipfile, Pipfile.lock|HIGH|
|**PNPM**|||||||
||Pnpm Lock||||||
||| Pnpm Lock |Node JS|npmjs | Files: pnpm-lock.yaml and package.json.|HIGH|
|**POETRY**|||||||
||Poetry Lock||||||
||| Poetry Lock |Python|pypi | Files: Poetry.lock, pyproject.toml|HIGH|
|**RUBYGEMS**|||||||
||Gemfile Lock||||||
||| Gemfile Lock |Ruby|RubyGems | File: Gemfile.lock.|HIGH|
||| Gemspec Parse |Ruby|RubyGems | File: A gemspec file (with .gemspec extension).|LOW|
||Gemspec Parse||||||
||| Gemspec Parse |Ruby|RubyGems | File: A gemspec file (with .gemspec extension).|LOW|
|**SBT**|||||||
||Sbt Native Inspector||||||
||| Sbt Native Inspector |Scala|Maven Central | File: build.sbt. Plugin: Dependency Graph|HIGH|
|**SWIFT**|||||||
||Swift Lock||||||
||| Swift Lock |Swift|Swift.org | File: Package.swift, Package.resolved|HIGH|
||| Swift CLI |Swift|Swift.org | File: Package.swift. Executables: swift.|HIGH|
||Swift CLI||||||
||| Swift CLI |Swift|Swift.org | File: Package.swift. Executables: swift.|HIGH|
|**XCODE**|||||||
||Xcode Workspace Lock||||||
||| Xcode Workspace Lock |Swift|GITHUB | Directory: *.xcworkspace|HIGH|
||| Xcode Project Lock |Swift|GITHUB | Directory: *.xcodeproj, Files: Package.resolved|HIGH|
||Xcode Project Lock||||||
||| Xcode Project Lock |Swift|GITHUB | Directory: *.xcodeproj, Files: Package.resolved|HIGH|
|**YARN**|||||||
||Yarn Lock||||||
||| Yarn Lock |Node JS|npmjs | Files: yarn.lock and package.json.|HIGH|


